import { DictionaryEntry } from './DictionaryEntry';

export type DictionaryEntriesOptions = {
  /**
   * Array of dictionary entries
   */
  readonly dictionaryEntries: readonly DictionaryEntry[];
};
