cp -a packages/algoliasearch/dist/algoliasearch-lite.umd* specs/fixtures/websites/algoliasearch-lite.com/.
cp -a packages/algoliasearch/dist/algoliasearch.umd* specs/fixtures/websites/algoliasearch.com/.
