/* eslint-disable functional/immutable-data */
/* eslint-disable import/no-commonjs */
/* eslint-disable max-len */

module.exports = {
  writerOpts: {
    headerPartial:
      '## [v{{version}}](https://github.com/algolia/algoliasearch-client-javascript/compare/{{previousTag}}...{{version}})\n',
  },
};
